@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* arith.c: arithmetic expansion */
-/* (C) 2007-2012 magicant */
+/* (C) 2007-2013 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -681,7 +681,8 @@ void parse_relational(evalinfo_T *info, value_T *result)
 		    case VT_INVALID:
 			result->type = VT_INVALID;
 			break;
-		    case VT_VAR:     assert(false);
+		    case VT_VAR:
+			assert(false);
 		}
 		break;
 	    default:
@@ -822,7 +823,12 @@ void parse_prefix(evalinfo_T *info, value_T *result)
 	case TT_MINUSMINUS:
 	    next_token(info);
 	    parse_prefix(info, result);
-	    if (result->type == VT_VAR) {
+	    if (posixly_correct) {
+		xerror(0, Ngt("arithmetic: operator `%ls' is not supported"),
+			(ttype == TT_PLUSPLUS) ? L"++" : L"--");
+		info->error = true;
+		result->type = VT_INVALID;
+	    } else if (result->type == VT_VAR) {
 		word_T saveword = result->v_var;
 		coerce_number(info, result);
 		do_increment_or_decrement(ttype, result);
@@ -892,7 +898,13 @@ void parse_postfix(evalinfo_T *info, value_T *result)
 	switch (info->token.type) {
 	    case TT_PLUSPLUS:
 	    case TT_MINUSMINUS:
-		if (result->type == VT_VAR) {
+		if (posixly_correct) {
+		    xerror(0,
+			    Ngt("arithmetic: operator `%ls' is not supported"),
+			    (info->token.type == TT_PLUSPLUS) ? L"++" : L"--");
+		    info->error = true;
+		    result->type = VT_INVALID;
+		} else if (result->type == VT_VAR) {
 		    word_T saveword = result->v_var;
 		    coerce_number(info, result);
 		    value_T value = *result;
