@@ -17,6 +17,7 @@
 
 
 #include "../common.h"
+#include "ulimit.h"
 #include <assert.h>
 #include <errno.h>
 #include <inttypes.h>
@@ -33,7 +34,6 @@
 #include "../builtin.h"
 #include "../exec.h"
 #include "../util.h"
-#include "ulimit.h"
 /* Including <stdint.h> is required before including <sys/resource.h> on
  * FreeBSD, but <stdint.h> is automatically included in <inttypes.h>. */
 /* On old Mac OS X, <sys/resource.h> depends on but does not include
@@ -53,6 +53,11 @@ struct resource {
     const char *description;
 };
 
+static int print_all_limits(bool soft);
+static void print_limit_value(bool soft, const struct rlimit *rlimit,
+	const struct resource *resource)
+    __attribute__((nonnull));
+
 #define RES(type,factor,desc) \
     (struct resource) { type, factor, desc, }
 
@@ -60,7 +65,7 @@ static const struct resource res_fsize = {
     RLIMIT_FSIZE, 512, Ngt("file size (blocks)")
 };
 
-static const struct xgetopt_T ulimit_options[] = {
+const struct xgetopt_T ulimit_options[] = {
     { L'H', L"hard", OPTARG_NONE, true, NULL, },
     { L'S', L"soft", OPTARG_NONE, true, NULL, },
     { L'a', L"all",  OPTARG_NONE, true, NULL, },
@@ -121,6 +126,7 @@ static const struct xgetopt_T ulimit_options[] = {
 int ulimit_builtin(int argc, void **argv)
 {
     enum { HARD = 1 << 0, SOFT = 1 << 1, } type = HARD | SOFT;
+    wchar_t resourceoption = L'\0';
     const struct resource *resource = &res_fsize;
     bool print_all = false;
 
@@ -137,119 +143,137 @@ int ulimit_builtin(int argc, void **argv)
 #endif
 	    default:
 		if (opt->ptr != NULL) {
+		    resourceoption = opt->shortopt;
 		    resource = opt->ptr;
 		    break;
 		} else {
-		    goto print_usage;
+		    return Exit_ERROR;
 		}
 	}
     }
 
-    struct rlimit rlimit;
-
     assert(type & (HARD | SOFT));
     if (print_all) {
-	for (opt = ulimit_options; opt->shortopt != L'\0'; opt++) {
-	    resource = opt->ptr;
-	    if (resource == NULL)
-		continue;
+	if (resourceoption != L'\0')
+	    return mutually_exclusive_option_error(L'a', resourceoption);
+	if (!validate_operand_count(argc - xoptind, 0, 0))
+	    return Exit_ERROR;
 
-	    if (getrlimit(resource->type, &rlimit) < 0) {
-		xerror(errno, Ngt("cannot get the current limit "
-				"for the resource type of `%s'"),
-				gt(resource->description));
-		continue;
-	    }
-
-	    rlim_t value = (type & SOFT) ? rlimit.rlim_cur : rlimit.rlim_max;
-	    xprintf(gt("-%lc: %-30s "),
-		    (wint_t) opt->shortopt, gt(resource->description));
-	    if (value == RLIM_INFINITY)
-		xprintf("%s\n", gt("unlimited"));
-	    else
-		xprintf("%ju\n", (uintmax_t) (value / resource->factor));
-	}
-	return yash_error_message_count == 0 ? Exit_SUCCESS : Exit_FAILURE;
+	return print_all_limits(type & SOFT);
     }
 
-    if (xoptind + 1 < argc)
-	goto print_usage;
+    if (!validate_operand_count(argc - xoptind, 0, 1))
+	return Exit_ERROR;
 
+    struct rlimit rlimit;
     if (getrlimit(resource->type, &rlimit) < 0) {
 	xerror(errno, Ngt("cannot get the current limit "
 			"for the resource type of `%s'"),
 			gt(resource->description));
 	return Exit_FAILURE;
     }
+
     if (xoptind == argc) {
-	/* print value */
-	rlim_t value = (type & SOFT) ? rlimit.rlim_cur : rlimit.rlim_max;
-	if (value == RLIM_INFINITY)
-	    xprintf("%s\n", gt("unlimited"));
-	else
-	    xprintf("%ju\n", (uintmax_t) (value / resource->factor));
+	print_limit_value(type & SOFT, &rlimit, resource);
 	return yash_error_message_count == 0 ? Exit_SUCCESS : Exit_FAILURE;
-    } else {
-	/* set value */
-	rlim_t value;
-	if (wcscmp(ARGV(xoptind), L"hard") == 0) {
-	    value = rlimit.rlim_max;
-	} else if (wcscmp(ARGV(xoptind), L"soft") == 0) {
-	    value = rlimit.rlim_cur;
-	} else if (wcscmp(ARGV(xoptind), L"unlimited") == 0) {
-	    value = RLIM_INFINITY;
-	} else if (iswdigit(ARGV(xoptind)[0])) {
-	    unsigned long v;
-
-	    if (!xwcstoul(ARGV(xoptind), 10, &v))
-		goto err_format;
-	    value = (rlim_t) v * resource->factor;
-	    if (value / resource->factor != v || value == RLIM_INFINITY
-		    || value == RLIM_SAVED_MAX || value == RLIM_SAVED_CUR) {
-		xerror(ERANGE, NULL);
-		return Exit_FAILURE;
-	    }
-	} else {
-	    goto err_format;
-	}
-	if (type & HARD)
-	    rlimit.rlim_max = value;
-	if (type & SOFT)
-	    rlimit.rlim_cur = value;
-	
-	/* check if soft limit exceeds hard limit */
-	if (rlimit.rlim_max != RLIM_INFINITY
-		&& rlimit.rlim_max != RLIM_SAVED_MAX
-		&& rlimit.rlim_max != RLIM_SAVED_CUR
-		&& (rlimit.rlim_cur == RLIM_INFINITY
-		    || (rlimit.rlim_cur != RLIM_SAVED_MAX
-			&& rlimit.rlim_cur != RLIM_SAVED_CUR
-			&& rlimit.rlim_cur > rlimit.rlim_max))) {
-	    xerror(0, Ngt("the soft limit cannot exceed the hard limit"));
-	    return Exit_FAILURE;
-	}
-
-	if (setrlimit(resource->type, &rlimit) < 0) {
-	    xerror(errno, Ngt("failed to set the limit"));
-	    return Exit_FAILURE;
-	}
-
-	return Exit_SUCCESS;
     }
 
+    /* parse the operand */
+    rlim_t value;
+    if (wcscmp(ARGV(xoptind), L"hard") == 0) {
+	value = rlimit.rlim_max;
+    } else if (wcscmp(ARGV(xoptind), L"soft") == 0) {
+	value = rlimit.rlim_cur;
+    } else if (wcscmp(ARGV(xoptind), L"unlimited") == 0) {
+	value = RLIM_INFINITY;
+    } else if (iswdigit(ARGV(xoptind)[0])) {
+	unsigned long v;
+
+	if (!xwcstoul(ARGV(xoptind), 10, &v))
+	    goto err_format;
+	value = (rlim_t) v * resource->factor;
+	if (value / resource->factor != v || value == RLIM_INFINITY
+		|| value == RLIM_SAVED_MAX || value == RLIM_SAVED_CUR) {
+	    xerror(ERANGE, NULL);
+	    return Exit_FAILURE;
+	}
+    } else {
+	goto err_format;
+    }
+
+    if (type & HARD)
+	rlimit.rlim_max = value;
+    if (type & SOFT)
+	rlimit.rlim_cur = value;
+    
+    /* check if soft limit does not exceed hard limit */
+    if (rlimit.rlim_max != RLIM_INFINITY
+	    && rlimit.rlim_max != RLIM_SAVED_MAX
+	    && rlimit.rlim_max != RLIM_SAVED_CUR
+	    && (rlimit.rlim_cur == RLIM_INFINITY
+		|| (rlimit.rlim_cur != RLIM_SAVED_MAX
+		    && rlimit.rlim_cur != RLIM_SAVED_CUR
+		    && rlimit.rlim_cur > rlimit.rlim_max))) {
+	xerror(0, Ngt("the soft limit cannot exceed the hard limit"));
+	return Exit_FAILURE;
+    }
+
+    if (setrlimit(resource->type, &rlimit) < 0) {
+	xerror(errno, Ngt("failed to set the limit"));
+	return Exit_FAILURE;
+    }
+
+    return Exit_SUCCESS;
+
 err_format:
     xerror(0, Ngt("`%ls' is not a valid integer"), ARGV(xoptind));
     return Exit_ERROR;
-print_usage:
-    fprintf(stderr, gt("Usage:  ulimit [option] [limit]\n"));
-    fprintf(stderr, gt("Available options: "));
-    fprintf(stderr, "-");
-    for (opt = ulimit_options; opt->shortopt != L'\0'; opt++)
-	if (opt->shortopt != L'-')
-	    fprintf(stderr, "%lc", (wint_t) opt->shortopt);
-    fprintf(stderr, "\n");
-    return Exit_ERROR;
 }
 
+/* Prints all current ulimit values to the standard output.
+ * Returns the exit status of the ulimit built-in. */
+int print_all_limits(bool soft)
+{
+    const struct xgetopt_T *opt;
+    for (opt = ulimit_options; opt->shortopt != L'\0'; opt++) {
+	const struct resource *resource = opt->ptr;
+	if (resource == NULL)
+	    continue;
+
+	struct rlimit rlimit;
+	if (getrlimit(resource->type, &rlimit) < 0) {
+	    xerror(errno, Ngt("cannot get the current limit "
+			    "for the resource type of `%s'"),
+			    gt(resource->description));
+	    continue;
+	}
+
+	xprintf(gt("-%lc: %-30s "),
+		(wint_t) opt->shortopt, gt(resource->description));
+	print_limit_value(soft, &rlimit, resource);
+    }
+    return yash_error_message_count == 0 ? Exit_SUCCESS : Exit_FAILURE;
+}
+
+void print_limit_value(bool soft, const struct rlimit *rlimit,
+	const struct resource *resource)
+{
+    rlim_t value = soft ? rlimit->rlim_cur : rlimit->rlim_max;
+    if (value == RLIM_INFINITY)
+	xprintf("%s\n", gt("unlimited"));
+    else
+	xprintf("%ju\n", (uintmax_t) (value / resource->factor));
+}
+
+#if YASH_ENABLE_HELP
+const char ulimit_help[] = Ngt(
+"set or print a resource limitation"
+);
+const char ulimit_syntax[] = Ngt(
+"\tulimit -a [-H|-S]\n"
+"\tulimit [-H|-S] [-efilnqrstuvx] [limit]\n"
+);
+#endif
+
 
 /* vim: set ts=8 sts=4 sw=4 noet tw=80: */
