@@ -58,29 +58,22 @@ static inline wchar_t get_euid_marker(void)
 
 /* An input function that inputs from a wide string.
  * `inputinfo' must be a pointer to a `struct input_wcs_info_T'.
- * Reads one line from `inputinfo->src' and appends it to the buffer `buf'.
- * If more string is available after reading the line, `inputinfo->src' is
- * updated so that it points to the character that should be read next. If no
- * more is available, `inputinfo->src' is assigned NULL. */
+ * Reads all characters from `inputinfo->src' and appends it to buffer `buf'.
+ * `inputinfo->src' is assigned NULL. */
 inputresult_T input_wcs(struct xwcsbuf_T *buf, void *inputinfo)
 {
     struct input_wcs_info_T *info = inputinfo;
     const wchar_t *src = info->src;
-    size_t count = 0;
 
     if (src == NULL)
 	return INPUT_EOF;
 
-    while (src[count] != L'\0' && src[count++] != L'\n');
+    bool isempty = (src[0] == L'\0');
 
-    if (count == 0) {
-	info->src = NULL;
-	return INPUT_EOF;
-    } else {
-	wb_ncat(buf, src, count);
-	info->src = &src[count];
-	return INPUT_OK;
-    }
+    wb_cat(buf, src);
+    info->src = NULL;
+
+    return isempty ? INPUT_EOF : INPUT_OK;
 }
 
 /* An input function that reads input from a file stream.
@@ -180,11 +173,11 @@ inputresult_T input_interactive(struct xwcsbuf_T *buf, void *inputinfo)
     struct input_interactive_info_T *info = inputinfo;
     struct promptset_T prompt;
 
-    if (info->prompttype == 1)
+    if (info->prompttype == 1) {
 	if (!posixly_correct)
-	    exec_variable_as_commands(L VAR_PROMPT_COMMAND, VAR_PROMPT_COMMAND);
-    if (info->prompttype == 1)
+	    exec_variable_as_auxiliary_(VAR_PROMPT_COMMAND);
 	check_mail();
+    }
     prompt = get_prompt(info->prompttype);
     if (do_job_control)
 	print_job_status_all();
