@@ -17,6 +17,7 @@
 
 
 #include "../common.h"
+#include "keymap.h"
 #include <assert.h>
 #include <errno.h>
 #if HAVE_GETTEXT
@@ -33,7 +34,6 @@
 #include "complete.h"
 #include "editing.h"
 #include "key.h"
-#include "keymap.h"
 #include "trie.h"
 
 
@@ -374,6 +374,18 @@ static int print_binding_main(
 static const char *get_command_name(le_command_func_T *command)
     __attribute__((nonnull,const));
 
+/* Options for the "bindkey" built-in. */
+const struct xgetopt_T bindkey_options[] = {
+    { L'v', L"vi-insert",  OPTARG_NONE, false, NULL, },
+    { L'a', L"vi-command", OPTARG_NONE, false, NULL, },
+    { L'e', L"emacs",      OPTARG_NONE, false, NULL, },
+    { L'l', L"list",       OPTARG_NONE, false, NULL, },
+#if YASH_ENABLE_HELP
+    { L'-', L"help",       OPTARG_NONE, false, NULL, },
+#endif
+    { L'\0', NULL, 0, false, NULL, },
+};
+
 /* The "bindkey" built-in, which accepts the following options:
  *  -v: select the "vi-insert" mode
  *  -a: select the "vi-command" mode
@@ -381,23 +393,12 @@ static const char *get_command_name(le_command_func_T *command)
  *  -l: list names of available commands */
 int bindkey_builtin(int argc, void **argv)
 {
-    static const struct xgetopt_T options[] = {
-	{ L'v', L"vi-insert",  OPTARG_NONE, false, NULL, },
-	{ L'a', L"vi-command", OPTARG_NONE, false, NULL, },
-	{ L'e', L"emacs",      OPTARG_NONE, false, NULL, },
-	{ L'l', L"list",       OPTARG_NONE, false, NULL, },
-#if YASH_ENABLE_HELP
-	{ L'-', L"help",       OPTARG_NONE, false, NULL, },
-#endif
-	{ L'\0', NULL, 0, false, NULL, },
-    };
-
     bool list = false;
     le_mode_id_T mode = LE_MODE_N;
 
     const struct xgetopt_T *opt;
     xoptind = 0;
-    while ((opt = xgetopt(argv, options, 0)) != NULL) {
+    while ((opt = xgetopt(argv, bindkey_options, 0)) != NULL) {
 	switch (opt->shortopt) {
 	    case L'a':  mode = LE_MODE_VI_COMMAND;  break;
 	    case L'e':  mode = LE_MODE_EMACS;       break;
@@ -408,42 +409,39 @@ int bindkey_builtin(int argc, void **argv)
 		return print_builtin_help(ARGV(0));
 #endif
 	    default:
-		goto print_usage;
+		return Exit_ERROR;
 	}
     }
 
     le_keymap_init();
 
     if (list) {
+	if (!validate_operand_count(argc - xoptind, 0, 0))
+	    return Exit_ERROR;
 	if (mode != LE_MODE_N) {
 	    xerror(0, Ngt("option combination is invalid"));
-	    goto print_usage;
+	    return Exit_ERROR;
 	}
 	return print_all_commands();
     }
 
     if (mode == LE_MODE_N) {
 	xerror(0, Ngt("no option is specified"));
-	goto print_usage;
+	return Exit_ERROR;
     }
 
-    if (xoptind == argc) {
-	/* print all key bindings */
-	le_mode_T *m = le_id_to_mode(mode);
-	return trie_foreachw(m->keymap, print_binding_main, m);
-    } else if (xoptind + 1 == argc) {
-	return print_binding(mode, ARGV(xoptind));
-    } else if (xoptind + 2 == argc) {
-	return set_key_binding(mode, ARGV(xoptind), ARGV(xoptind + 1));
-    } else {
-	xerror(0, Ngt("too many operands are specified"));
-	goto print_usage;
+    switch (argc - xoptind) {
+	case 0:;
+	    /* print all key bindings */
+	    le_mode_T *m = le_id_to_mode(mode);
+	    return trie_foreachw(m->keymap, print_binding_main, m);
+	case 1:
+	    return print_binding(mode, ARGV(xoptind));
+	case 2:
+	    return set_key_binding(mode, ARGV(xoptind), ARGV(xoptind + 1));
+	default:
+	    return too_many_operands_error(2);
     }
-
-print_usage:
-    fprintf(stderr, gt("Usage:  bindkey -aev [keyseq [command]]\n"
-		       "        bindkey -l\n"));
-    return Exit_ERROR;
 }
 
 /* Prints all available commands to the standard output. */
@@ -564,5 +562,15 @@ const char *get_command_name(le_command_func_T *command)
     return NULL;
 }
 
+#if YASH_ENABLE_HELP
+const char bindkey_help[] = Ngt(
+"set or print key bindings for line-editing"
+);
+const char bindkey_syntax[] = Ngt(
+"\tbindkey -aev [key_sequence [command]]\n"
+"\tbindkey -l\n"
+);
+#endif
+
 
 /* vim: set ts=8 sts=4 sw=4 noet tw=80: */
