@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* strbuf.h: modifiable string buffer */
-/* (C) 2007-2009 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -89,7 +89,15 @@ extern xstrbuf_T *sb_ccat_repeat(xstrbuf_T *buf, char c, size_t n)
 extern _Bool sb_wccat(
 	xstrbuf_T *restrict buf, wchar_t c, mbstate_t *restrict ps)
     __attribute__((nonnull));
-extern wchar_t *sb_wcscat(xstrbuf_T *restrict buf,
+extern wchar_t *sb_wcsncat(xstrbuf_T *restrict buf,
+	const wchar_t *restrict s, size_t n, mbstate_t *restrict ps)
+    __attribute__((nonnull));
+#if HAVE_WCSNRTOMBS
+static inline
+#else
+extern
+#endif
+wchar_t *sb_wcscat(xstrbuf_T *restrict buf,
 	const wchar_t *restrict s, mbstate_t *restrict ps)
     __attribute__((nonnull));
 extern int sb_vprintf(
@@ -148,8 +156,6 @@ static inline xwcsbuf_T *wb_remove(xwcsbuf_T *buf, size_t i, size_t n)
     __attribute__((nonnull));
 extern xwcsbuf_T *wb_wccat(xwcsbuf_T *buf, wchar_t c)
     __attribute__((nonnull));
-extern xwcsbuf_T *wb_wccat_repeat(xwcsbuf_T *buf, wchar_t c, size_t n)
-    __attribute__((nonnull));
 extern char *wb_mbscat(xwcsbuf_T *restrict buf, const char *restrict s)
     __attribute__((nonnull));
 extern int wb_vwprintf(
@@ -161,26 +167,32 @@ extern int wb_wprintf(
 
 extern char *malloc_wcsntombs(const wchar_t *s, size_t n)
     __attribute__((nonnull,malloc,warn_unused_result));
-extern char *malloc_wcstombs(const wchar_t *s)
+#if HAVE_WCSNRTOMBS
+static inline
+#else
+extern
+#endif
+char *malloc_wcstombs(const wchar_t *s)
     __attribute__((nonnull,malloc,warn_unused_result));
 static inline char *realloc_wcstombs(wchar_t *s)
     __attribute__((nonnull,malloc,warn_unused_result));
-extern wchar_t *malloc_mbsntowcs(const char *s, size_t n)
-    __attribute__((nonnull,malloc,warn_unused_result));
 extern wchar_t *malloc_mbstowcs(const char *s)
     __attribute__((nonnull,malloc,warn_unused_result));
 static inline wchar_t *realloc_mbstowcs(char *s)
     __attribute__((nonnull,malloc,warn_unused_result));
 
-static inline char *malloc_vprintf(const char *format, va_list ap)
+extern char *malloc_vprintf(const char *format, va_list ap)
     __attribute__((nonnull(1),malloc,warn_unused_result,format(printf,1,0)));
 extern char *malloc_printf(const char *format, ...)
     __attribute__((nonnull(1),malloc,warn_unused_result,format(printf,1,2)));
-static inline wchar_t *malloc_vwprintf(const wchar_t *format, va_list ap)
+extern wchar_t *malloc_vwprintf(const wchar_t *format, va_list ap)
     __attribute__((nonnull(1),malloc,warn_unused_result));
 extern wchar_t *malloc_wprintf(const wchar_t *format, ...)
     __attribute__((nonnull(1),malloc,warn_unused_result));
 
+extern wchar_t *joinwcsarray(void *const *array, const wchar_t *padding)
+    __attribute__((malloc,warn_unused_result,nonnull));
+
 
 /* Frees the specified multibyte string buffer. The contents are lost. */
 void sb_destroy(xstrbuf_T *buf)
@@ -285,6 +297,14 @@ xstrbuf_T *sb_remove(xstrbuf_T *buf, size_t i, size_t n)
     return sb_replace(buf, i, n, "", 0);
 }
 
+#if HAVE_WCSNRTOMBS
+wchar_t *sb_wcscat(xstrbuf_T *restrict buf,
+	const wchar_t *restrict s, mbstate_t *restrict ps)
+{
+    return sb_wcsncat(buf, s, Size_max, ps);
+}
+#endif
+
 /* Frees the specified wide string buffer. The contents are lost. */
 void wb_destroy(xwcsbuf_T *buf)
 {
@@ -389,6 +409,13 @@ xwcsbuf_T *wb_remove(xwcsbuf_T *buf, size_t i, size_t n)
     return wb_replace(buf, i, n, L"", 0);
 }
 
+#if HAVE_WCSNRTOMBS
+char *malloc_wcstombs(const wchar_t *s)
+{
+    return malloc_wcsntombs(s, Size_max);
+}
+#endif
+
 /* Converts the specified wide string into a newly malloced multibyte string and
  * frees the original wide string.
  * Returns NULL on error. The wide string is freed anyway.
@@ -410,24 +437,6 @@ wchar_t *realloc_mbstowcs(char *s)
     return result;
 }
 
-/* Returns the result of `vsprintf' as a newly malloced string. */
-char *malloc_vprintf(const char *format, va_list ap)
-{
-    xstrbuf_T buf;
-    sb_init(&buf);
-    sb_vprintf(&buf, format, ap);
-    return sb_tostr(&buf);
-}
-
-/* Returns the result of `vswprintf' as a newly malloced string. */
-wchar_t *malloc_vwprintf(const wchar_t *format, va_list ap)
-{
-    xwcsbuf_T buf;
-    wb_init(&buf);
-    wb_vwprintf(&buf, format, ap);
-    return wb_towcs(&buf);
-}
-
 
 #undef Size_max
 
