@@ -58,7 +58,7 @@ echo "PATH=$PATH"
 
 export INVOKE TESTEE TESTTMP
 export LC_MESSAGES=POSIX LC_CTYPE="${LC_ALL-${LC_CTYPE-${LANG}}}" LANG=POSIX
-unset ENV HISTFILE HISTSIZE MAIL MAILCHECK MAILPATH IFS LC_ALL
+unset CDPATH ENV HISTFILE HISTSIZE MAIL MAILCHECK MAILPATH IFS LC_ALL
 unset YASH_AFTER_CD COMMAND_NOT_FOUND_HANDLER HANDLED PROMPT_COMMAND
 umask u=rwx,go=
 
@@ -127,8 +127,8 @@ diffresult() {
 }
 
 echo
-echo "Testing ${TESTEE:=../yash} for ${TEST_ITEMS:=*.tst}"
-echo "Testing ${TESTEE:=../yash} for ${TEST_ITEMS:=*.tst}" >&3
+printf 'Testing %s for %s\n' "${TESTEE:=../yash}" "${TEST_ITEMS:=*.tst}"
+printf 'Testing %s for %s\n' "${TESTEE:=../yash}" "${TEST_ITEMS:=*.tst}" >&3
 
 failed=0
 for x in $TEST_ITEMS
@@ -192,11 +192,12 @@ echo
 
 if [ 0 -eq $failed ]
 then
-    echo "All test(s) completed successfully."
-    echo "All test(s) completed successfully." >&3
+    printf '%s passed all of %s.\n' "${TESTEE}" "${TEST_ITEMS}"
+    printf '%s passed all of %s.\n' "${TESTEE}" "${TEST_ITEMS}" >&3
 else
-    echo "${failed} test(s) failed."
-    echo "${failed} test(s) failed. See test.log for more info." >&3
+    printf '%s failed %d of %s.\n' "${TESTEE}" "${failed}" "${TEST_ITEMS}"
+    printf '%s failed %d of %s.  See test.log for more info.\n' \
+	"${TESTEE}" "${failed}" "${TEST_ITEMS}" >&3
     false
 fi
 
