@@ -24,7 +24,28 @@
 #define Size_max ((size_t) -1)  // = SIZE_MAX
 
 
-/********** General Functions **********/
+/********** Miscellaneous Functions **********/
+
+extern size_t addmul(size_t mainsize, size_t count, size_t elemsize)
+    __attribute__((pure));
+static inline int xunsetenv(const char *name)
+    __attribute__((nonnull));
+
+/* Removes the environment variable of the specified name.
+ * This function wraps the `unsetenv' function, which has an incompatible
+ * prototype on some old environments. */
+int xunsetenv(const char *name)
+{
+#if UNSETENV_RETURNS_INT
+    return unsetenv(name);
+#else
+    unsetenv(name);
+    return 0;
+#endif
+}
+
+
+/********** Memory Functions **********/
 
 static inline void *xcalloc(size_t nmemb, size_t size)
     __attribute__((malloc,warn_unused_result));
@@ -41,12 +62,8 @@ static inline void *xreallocn(void *ptr, size_t count, size_t elemsize)
 static inline void *xreallocs(void *ptr,
 	size_t mainsize, size_t count, size_t elemsize)
     __attribute__((malloc,warn_unused_result));
-extern size_t addmul(size_t mainsize, size_t count, size_t elemsize)
-    __attribute__((pure));
 extern void alloc_failed(void)
     __attribute__((noreturn));
-static inline int xunsetenv(const char *name)
-    __attribute__((nonnull));
 
 /* Attempts `calloc' and aborts the program on failure. */
 void *xcalloc(size_t nmemb, size_t size)
@@ -103,19 +120,6 @@ void *xreallocs(void *ptr, size_t mainsize, size_t count, size_t elemsize)
     return xrealloc(ptr, addmul(mainsize, count, elemsize));
 }
 
-/* Removes the environment variable of the specified name.
- * This function wraps the `unsetenv' function, which has an incompatible
- * prototype on some old environments. */
-int xunsetenv(const char *name)
-{
-#if UNSETENV_RETURNS_INT
-    return unsetenv(name);
-#else
-    unsetenv(name);
-    return 0;
-#endif
-}
-
 
 /********** String Utilities **********/
 
@@ -210,33 +214,6 @@ extern _Bool xprintf(const char *restrict format, ...)
     __attribute__((format(printf,1,2)));
 
 
-/********** xgetopt **********/
-
-extern wchar_t *xoptarg;
-extern int xoptind;
-
-enum xgetoptopt_T {
-    XGETOPT_POSIX = 1 << 0,
-    XGETOPT_DIGIT = 1 << 1,
-};
-enum optarg_T {
-    OPTARG_NONE, OPTARG_REQUIRED, OPTARG_OPTIONAL,
-};
-struct xgetopt_T {
-    wchar_t shortopt;
-    const wchar_t *longopt;
-    enum optarg_T optarg;
-    _Bool posix;
-    void *ptr;
-};
-
-extern struct xgetopt_T *xgetopt(
-	void **restrict argv,
-	const struct xgetopt_T *restrict opts,
-	enum xgetoptopt_T opt)
-    __attribute__((nonnull));
-
-
 #undef Size_max
 
 #endif /* YASH_UTIL_H */
