@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* yash.c: basic functions of the shell */
-/* (C) 2007-2012 magicant */
+/* (C) 2007-2013 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,6 +59,7 @@ static struct input_file_info_T *new_input_file_info(int fd, size_t bufsize)
     __attribute__((malloc,warn_unused_result));
 static void execute_profile(const wchar_t *profile);
 static void execute_rcfile(const wchar_t *rcfile);
+static void execute_file(const wchar_t *path);
 static void print_help(void);
 static void print_version(void);
 
@@ -74,6 +75,9 @@ pid_t shell_pid;
  * In a job-controlled subshell, this value is set to the subshell's pgid. */
 pid_t shell_pgid;
 
+/* Set to true when the shell has been initialized. */
+bool shell_initialized;
+
 /* If this flag is true when the "exit" built-in is invoked, the -f option is
  * assumed to be specified. */
 static bool forceexit;
@@ -231,11 +235,14 @@ int main(int argc, char **argv)
 	if (getuid() == geteuid() && getgid() == getegid())
 	    execute_rcfile(options.rcfile);
 
-    if (shopt_cmdline) {
+    shell_initialized = true;
+
+    if (shopt_cmdline)
 	exec_wcs(input.command, inputname, true);
-    } else {
-	exec_input(input.fd, inputname, is_interactive, true, true);
-    }
+    else
+	exec_input(input.fd, inputname, XIO_SUBST_ALIAS | XIO_FINALLY_EXIT |
+		(is_interactive ? XIO_INTERACTIVE : 0));
+
     assert(false);
 }
 
@@ -253,65 +260,61 @@ struct input_file_info_T *new_input_file_info(int fd, size_t bufsize)
 /* Executes "$HOME/.yash_profile". */
 void execute_profile(const wchar_t *profile)
 {
-    char *path;
-
     if (profile != NULL) {
-	path = malloc_wcstombs(profile);
-    } else {
-	wchar_t *wpath =
-	    parse_and_expand_string(L"$HOME/.yash_profile", NULL, false);
-	if (wpath == NULL)
-	    return;
-	path = realloc_wcstombs(wpath);
-    }
-    if (path == NULL)
+	execute_file(profile);
 	return;
-
-    int fd = move_to_shellfd(open(path, O_RDONLY));
-    if (fd >= 0) {
-	exec_input(fd, path, false, true, false);
-	remove_shellfd(fd);
-	xclose(fd);
     }
+
+    wchar_t *path =
+	parse_and_expand_string(L"$HOME/.yash_profile", NULL, false);
+    execute_file(path);
     free(path);
 }
 
 /* Executes the initialization file.
  * `rcfile' is the filename to source.
- * If `rcfile' is NULL, it defaults to "$HOME/.yashrc" or "$ENV". */
+ * If `rcfile' is NULL, it defaults to "$HOME/.yashrc".
+ * In the POSIXly-correct mode, `rcfile' is ignored and "$ENV" is used. */
 void execute_rcfile(const wchar_t *rcfile)
 {
-    char *path;
-
     if (posixly_correct) {
 	const wchar_t *env = getvar(L VAR_ENV);
 	if (env == NULL)
 	    return;
-	wchar_t *wpath = parse_and_expand_string(env, "$ENV", false);
-	if (wpath == NULL)
-	    return;
-	path = realloc_wcstombs(wpath);
-    } else {
-	if (rcfile != NULL) {
-	    path = malloc_wcstombs(rcfile);
-	} else {
-	    wchar_t *wpath =
-		parse_and_expand_string(L"$HOME/.yashrc", NULL, false);
-	    if (wpath == NULL)
-		return;
-	    path = realloc_wcstombs(wpath);
-	}
+	wchar_t *path = parse_and_expand_string(env, "$ENV", false);
+	execute_file(path);
+	free(path);
+	return;
     }
+
+    if (rcfile != NULL) {
+	execute_file(rcfile);
+	return;
+    }
+
+    wchar_t *path = parse_and_expand_string(L"$HOME/.yashrc", NULL, false);
+    execute_file(path);
+    free(path);
+}
+
+/* Executes the specified file if `path' is non-NULL. */
+void execute_file(const wchar_t *path)
+{
     if (path == NULL)
 	return;
 
-    int fd = move_to_shellfd(open(path, O_RDONLY));
+    char *mbspath = malloc_wcstombs(path);
+    if (mbspath == NULL)
+	return;
+
+    int fd = move_to_shellfd(open(mbspath, O_RDONLY));
     if (fd >= 0) {
-	exec_input(fd, path, false, true, false);
+	exec_input(fd, mbspath, XIO_SUBST_ALIAS);
 	remove_shellfd(fd);
 	xclose(fd);
     }
-    free(path);
+
+    free(mbspath);
 }
 
 /* Exits the shell with the specified exit status.
@@ -441,22 +444,21 @@ void exec_wcs(const wchar_t *code, const char *name, bool finally_exit)
 
 /* Parses the input from the specified file descriptor and executes commands.
  * The file descriptor must be either STDIN_FILENO or a shell FD. If the file
- * descriptor is STDIN_FILENO, `finally_exit' must be true.
+ * descriptor is STDIN_FILENO, XIO_FINALLY_EXIT must be specified in `options'.
  * If `name' is non-NULL, it is printed in an error message on syntax error.
- * If `interactive' is true, the input is considered interactive.
+ * If XIO_INTERACTIVE is specified, the input is considered interactive.
  * If there are no commands in the input, `laststatus' is set to zero. */
-void exec_input(int fd, const char *name,
-	bool interactive, bool enable_alias, bool finally_exit)
+void exec_input(int fd, const char *name, exec_input_options_T options)
 {
     struct parseparam_T pinfo = {
 	.print_errmsg = true,
 	.enable_verbose = true,
 #if YASH_ENABLE_ALIAS
-	.enable_alias = enable_alias,
+	.enable_alias = options & XIO_SUBST_ALIAS,
 #endif
 	.filename = name,
 	.lineno = 1,
-	.interactive = interactive,
+	.interactive = options & XIO_INTERACTIVE,
     };
     struct input_interactive_info_T intrinfo;
     struct input_file_info_T *inputinfo;
@@ -466,7 +468,7 @@ void exec_input(int fd, const char *name,
     else
 	inputinfo = new_input_file_info(fd, BUFSIZ);
 
-    if (interactive) {
+    if (pinfo.interactive) {
 	intrinfo.fileinfo = inputinfo;
 	intrinfo.prompttype = 1;
 	pinfo.input = input_interactive;
@@ -475,18 +477,14 @@ void exec_input(int fd, const char *name,
 	pinfo.input = input_file;
 	pinfo.inputinfo = inputinfo;
     }
-    parse_and_exec(&pinfo, finally_exit);
+    parse_and_exec(&pinfo, options & XIO_FINALLY_EXIT);
 
     assert(inputinfo != stdin_input_file_info);
     free(inputinfo);
-
-#if !YASH_ENABLE_ALIAS
-    (void) enable_alias;  // suppress compiler warning
-#endif
 }
 
-/* Parses the input using the specified `parseinfo_T' and executes commands.
- * If no commands were executed, `laststatus' is set to zero. */
+/* Parses the input using the specified `parseparam_T' and executes commands.
+ * If no commands were executed, `laststatus' is set to Exit_SUCCESS. */
 void parse_and_exec(parseparam_T *pinfo, bool finally_exit)
 {
     bool executed = false;
@@ -519,21 +517,19 @@ void parse_and_exec(parseparam_T *pinfo, bool finally_exit)
 		}
 		break;
 	    case PR_EOF:
-		if (shopt_ignoreeof && input_is_interactive_terminal(pinfo)) {
-		    fprintf(stderr, gt("Use `exit' to leave the shell.\n"));
-		    break;
-		}
 		if (!executed)
 		    laststatus = Exit_SUCCESS;
-		if (finally_exit) {
+		if (!finally_exit)
+		    goto out;
+		if (shopt_ignoreeof && input_is_interactive_terminal(pinfo)) {
+		    fprintf(stderr, gt("Use `exit' to leave the shell.\n"));
+		} else {
 		    wchar_t argv0[] = L"EOF";
 		    exit_builtin(1, (void *[]) { argv0, NULL });
-		} else {
-		    goto out;
 		}
 		break;
 	    case PR_SYNTAX_ERROR:
-		if (!is_interactive_now)
+		if (shell_initialized && !is_interactive_now)
 		    exit_shell_with_status(Exit_SYNERROR);
 		if (!pinfo->interactive) {
 		    laststatus = Exit_SYNERROR;
